package com.bomostory.sceneeditmodule.cover

import android.content.Context
import android.graphics.Bitmap
import android.support.constraint.ConstraintLayout
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.RadioButton
import com.bomostory.sceneeditmodule.SceneDrawer
import com.bomostory.sceneeditmodule.basicdata.Project
import com.bomostory.sceneeditmodule.utils.Utils
import com.bomostory.sceneeditmodule.utils.Utils.crop
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.item_front_cover_scene_chooser.view.*
import kotlinx.android.synthetic.main.view_front_cover_scene_chooser.view.*

class FrontCoverSceneChooserView @JvmOverloads constructor(
        context: Context,
        attrs: AttributeSet? = null,
        defStyleAttr: Int = 0
) : ConstraintLayout(context, attrs, defStyleAttr) {

    private val alignStyleStringMap = HashMap<AlignStyle, Int>().apply {
        put(AlignStyle.Center, R.string.front_cover_scene_choose_dialog_align_center)
        put(AlignStyle.Left, R.string.front_cover_scene_choose_dialog_align_left)
        put(AlignStyle.Right, R.string.front_cover_scene_choose_dialog_align_right)
    }
    var selected: Int = 0
        private set
    private var project: Project? = null
        set(value) {
            field = value
            update()
        }
    val alignStyle: AlignStyle
        get() {
            val position = spinner_frontCoverSceneChooserDialog_align.selectedItemPosition
            return AlignStyle.values()[position]
        }
    var title: String = ""
        set(value) {
            tv_frontCoverSceneChooserDialog_coverTitle.text = value
        }
    var author: String = ""
        set(value) {
            tv_frontCoverSceneChooserDialog_coverAuthor.text = value
        }
    var onApply: View.OnClickListener? = null
    var onDiscard: View.OnClickListener? = null

    private val dataList = ArrayList<Data>()
    private val adapter = Adapter(dataList)
    private var lastSelectedData: Data? = null

    init {
        inflate(context, R.layout.view_front_cover_scene_chooser, this)
        setOnClickListener {
            // do nothing
        }
        btn_frontCoverSceneChooserDialog_apply.setOnClickListener { onApply?.onClick(it) }
        btn_frontCoverSceneChooserDialog_discard.setOnClickListener { onDiscard?.onClick(it) }
        recyclerView_frontCoverSceneChooserDialog_sceneList.layoutManager = LinearLayoutManager(context).apply {
            orientation = LinearLayoutManager.VERTICAL
        }
        recyclerView_frontCoverSceneChooserDialog_sceneList.adapter = adapter

        // align style spinner
        val itemLayoutResId = android.R.layout.simple_dropdown_item_1line
        spinner_frontCoverSceneChooserDialog_align.adapter = ArrayAdapter<String>(context, itemLayoutResId).apply {
            AlignStyle.values().forEach {
                val resId = alignStyleStringMap[it] ?: return@forEach
                val s = context.getString(resId)
                add(s)
            }
        }
        spinner_frontCoverSceneChooserDialog_align.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(p0: AdapterView<*>?) {}

            override fun onItemSelected(p0: AdapterView<*>?, p1: View?, p2: Int, p3: Long) {
                updateCover()
            }
        }
    }

    fun setProjectAndSelected(project: Project, selected: Int) {
        this.selected = selected
        this.project = project
    }

    private fun update() {
        val p = project ?: return
        dataList.clear()
        var i = 0
        p.story?.scenes?.forEach { scene ->
            val width = context.resources.getDimension(R.dimen.share_dialog_screenshot_width).toInt()
            val height = context.resources.getDimension(R.dimen.share_dialog_screenshot_height).toInt()
            val bitmap = SceneDrawer.drawScene(context, scene, 0, width, height) ?: return@forEach
            val isSelected = i == selected
            val name = Utils.generateSceneName(i)
            val onSelectedListener = object : OnSelectedListener {
                override fun onSelected(data: Data) {
                    lastSelectedData?.isSelected = false
                    lastSelectedData = data
                    lastSelectedData?.isSelected = true
                    project?.story?.scenes?.also { scenes ->
                        selected = dataList.indexOf(data)
                        updateCover()
                    }
                }
            }
            val data = Data(isSelected, bitmap, name, onSelectedListener)
            dataList.add(data)
            i++
        }
        adapter.notifyDataSetChanged()
        updateCover()
    }

    private fun updateCover() {
        val scenes = project?.story?.scenes ?: return
        val scene = scenes[selected]
        val width = context.resources.getDimension(R.dimen.share_dialog_screenshot_width).toInt()
        val height = context.resources.getDimension(R.dimen.share_dialog_screenshot_height).toInt()
        val bitmap = SceneDrawer.drawScene(context, scene, 0, width, height)?.crop(alignStyle)
                ?: return
        iv_frontCoverSceneChooserDialog_cover.setImageBitmap(bitmap)
    }

    private inner class Adapter(private val dataList: List<Data>) : RecyclerView.Adapter<ViewHolder>() {

        private var lastCheckedRadioButton: RadioButton? = null

        private val onSelectedListener = object : ViewHolder.OnSelectedListener {
            override fun onSelected(radioButton: RadioButton) {
                lastCheckedRadioButton?.isChecked = false
                lastCheckedRadioButton = radioButton
                lastCheckedRadioButton?.isChecked = true
            }
        }

        override fun onCreateViewHolder(parent: ViewGroup, type: Int): ViewHolder {
            val v1 = LayoutInflater.from(parent.context).inflate(R.layout.item_front_cover_scene_chooser, parent, false)
            return ViewHolder(v1)
        }

        override fun getItemCount(): Int {
            return dataList.size
        }

        override fun onBindViewHolder(p0: ViewHolder, p1: Int) {
            val data = dataList[p1]
            p0.bind(data, onSelectedListener)
        }

        override fun onViewRecycled(holder: ViewHolder) {
            super.onViewRecycled(holder)
            holder.itemView.rb_frontCoverSceneChooserItem.setOnCheckedChangeListener(null)
        }
    }

    private class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        fun bind(data: Data, onSelectedListener: OnSelectedListener) {
            if (data.isSelected) {
                onSelectedListener.onSelected(itemView.rb_frontCoverSceneChooserItem)
            }
            itemView.rb_frontCoverSceneChooserItem.setOnCheckedChangeListener { _, b ->
                if (b) {
                    onSelectedListener.onSelected(itemView.rb_frontCoverSceneChooserItem)
                    data.onSelectedListener.onSelected(data)
                }
            }
            itemView.iv_frontCoverSceneChooserItem_screenShot.setImageBitmap(data.screenshot)
            itemView.tv_frontCoverSceneChooserItem_name.text = data.name
            itemView.setOnClickListener {
                itemView.rb_frontCoverSceneChooserItem.isChecked = true
            }
            itemView.rb_frontCoverSceneChooserItem.isChecked = data.isSelected
        }

        interface OnSelectedListener {
            fun onSelected(radioButton: RadioButton)
        }
    }

    private data class Data(
            var isSelected: Boolean,
            val screenshot: Bitmap,
            val name: String,
            val onSelectedListener: OnSelectedListener
    )

    private interface OnSelectedListener {
        fun onSelected(data: Data)
    }
}