package com.bomostory.sceneeditmodule.screen.view

import android.content.Context
import android.graphics.*
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.widget.ImageView
import com.bomostory.sceneeditmodule.DialogueDrawer
import com.bomostory.sceneeditmodule.basicdata.Actor
import pl.droidsonroids.gif.GifDrawable
import kotlin.math.max

class ActorView : ImageView {

    var gifDrawable: GifDrawable? = null

    var data: Actor? = null
        set(value) {
            field = value
            value?.let {
                if(!it.isDialogue) {
                    if (it.resourcePath.toLowerCase().endsWith(".gif")) {
                        gifDrawable = GifDrawable(it.resourcePath)
                        setImageDrawable(gifDrawable)
                        gifDrawable?.start()
                    } else {
                        setImageDrawable(Drawable.createFromPath(it.resourcePath))
                    }
                }
                if (it.isMirror && !it.isDialogue) {
                    this.scaleX = -1f
                    this.scaleY = 1f
                }
                this.alpha = it.opacity
            }
        }
    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        data?.let {
            if (it.isDialogue) {
                canvas?.drawBitmap(DialogueDrawer.drawDialogue(context, it), 0f, 0f, null)
            }
        }
    }

    fun GifDrawable.getBitmapAt(milliseconds: Int): Bitmap = seekToPositionAndGet(if (loopCount == 0 || milliseconds < duration * loopCount) max(0, milliseconds % duration) else (duration))

    fun isGif(): Boolean {
        return gifDrawable != null
    }

    fun isGifPlaying(): Boolean {
        if (isGif()) {
            return gifDrawable!!.isPlaying
        }
        return false
    }

    fun startGifAnimation() {
        if (isGif()) {
            setImageDrawable(gifDrawable)
            gifDrawable!!.start()
        }
    }

    fun stopGifAnimation() {
        if (isGif()) {
            gifDrawable!!.stop()
        }
    }

    fun getBitmapAt(milliseconds: Int): Bitmap? {
        if (isGif()) {
            return gifDrawable!!.getBitmapAt(milliseconds)
        } else if (drawable is BitmapDrawable) {
            val bitmapDrawable = drawable as BitmapDrawable
            return bitmapDrawable.bitmap
        }
        return null
    }
}